@@ -31,8 +31,8 @@ except ImportError:  # Python 3.7 and lower
     from importlib_metadata import files, PackageNotFoundError  # type: ignore
 
 # local imports
-from maestral.utils.appdirs import get_home_dir, get_conf_path, get_data_path
-from maestral.constants import BUNDLE_ID
+from .utils.appdirs import get_home_dir, get_conf_path, get_data_path
+from .constants import BUNDLE_ID, ENV
 
 
 class SupportedImplementations(Enum):
@@ -133,9 +133,11 @@ class AutoStartLaunchd(AutoStartBase):
 
     :param bundle_id: Bundle ID for the, e.g., "com.google.calendar".
     :param start_cmd: Absolute path to executable and optional program arguments.
+    :param kwargs: Additional key, value pairs to add to plist. Values may be strings,
+        booleans, lists or dictionaries.
     """
 
-    def __init__(self, bundle_id: str, start_cmd: str) -> None:
+    def __init__(self, bundle_id: str, start_cmd: str, **kwargs) -> None:
 
         super().__init__()
         filename = bundle_id + ".plist"
@@ -149,6 +151,7 @@ class AutoStartLaunchd(AutoStartBase):
 
         self.plist_dict["Label"] = str(bundle_id)
         self.plist_dict["ProgramArguments"] = shlex.split(start_cmd)
+        self.plist_dict.update(kwargs)
 
     def enable(self) -> None:
         os.makedirs(self.path, exist_ok=True)
@@ -176,9 +179,9 @@ class AutoStartXDGDesktop(AutoStartBase):
 
     https://specifications.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html
 
-    :param Name: Name of application.
-    :param Exec: Executable on $PATH or absolute path to executable and optional program
-        arguments.
+    :param app_name: Name of application.
+    :param start_cmd: Executable on $PATH or absolute path to executable and optional
+        program arguments.
     :param filename: Name of desktop entry file. If not given, the application name will
         be used.
     :param kwargs: Additional key, value pairs to be used in the desktop entries. Values
@@ -297,18 +300,10 @@ class AutoStart:
         bundle_id = f"{BUNDLE_ID}-daemon.{config_name}"
 
         if self.implementation == SupportedImplementations.launchd:
-            self._impl = AutoStartLaunchd(bundle_id, start_cmd)
-
-        elif self.implementation == SupportedImplementations.xdg_desktop:
-            self._impl = AutoStartXDGDesktop(
-                filename=f"maestral-{config_name}.desktop",
-                app_name="Maestral",
-                start_cmd=start_cmd,
-                TryExec=self.maestral_path,
-                Icon="maestral",
-                Terminal="false",
-                GenericName="File Synchronizer",
-                Comment="Sync your files with Dropbox",
+            self._impl = AutoStartLaunchd(
+                bundle_id,
+                start_cmd,
+                EnvironmentVariables=ENV,
             )
 
         elif self.implementation == SupportedImplementations.systemd:
@@ -328,6 +323,7 @@ class AutoStart:
                     "WatchdogSec": "30",
                     "ExecStop": f"{self.maestral_path} stop -c %i",
                     "ExecStopPost": f'/usr/bin/env bash -c "{notify_failure}"',
+                    "Environment": " ".join(f"{k}={v}" for k, v in ENV.items()),
                 },
             )
 
@@ -336,11 +332,12 @@ class AutoStart:
 
     @property
     def enabled(self) -> bool:
-        """True if autostart is enabled."""
+        """True if autostart is enabled, False otherwise."""
         return self._impl.enabled
 
     @enabled.setter
     def enabled(self, value: bool) -> None:
+        """Setter: enabled."""
         if value:
             self.enable()
         else:
@@ -348,13 +345,10 @@ class AutoStart:
 
     def toggle(self) -> None:
         """Toggles autostart on or off."""
-        if self.enabled:
-            self.disable()
-        else:
-            self.enable()
+        self.enabled = not self.enabled
 
     def enable(self) -> None:
-        """Setter: True if autostart is enabled."""
+        """Enable autostart."""
 
         if self.enabled:
             return
@@ -365,7 +359,7 @@ class AutoStart:
             raise OSError("Could not find path of maestral executable")
 
     def disable(self) -> None:
-        """Setter: True if autostart is enabled."""
+        """Disable autostart."""
 
         if not self.enabled:
             return
