@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 """
-This file defines the functions to configure and interact with Maestral from the command
-line. Some imports are deferred to the functions that required them in order to reduce
-the startup time of individual CLI commands.
+This module defines the functions to configure and interact with Maestral from the
+command line. Some imports are deferred to the functions that required them in order to
+reduce the startup time of individual CLI commands.
 """
 
 # system imports
@@ -21,6 +21,7 @@ from . import __version__
 from .utils import cli
 
 if TYPE_CHECKING:
+    from click.shell_completion import CompletionItem
     from datetime import datetime
     from .main import Maestral
     from .daemon import MaestralProxy
@@ -189,12 +190,13 @@ def check_for_fatal_errors(m: Union["MaestralProxy", "Maestral"]) -> bool:
     """
 
     import textwrap
+    import shutil
 
     maestral_err_list = m.fatal_errors
 
     if len(maestral_err_list) > 0:
 
-        width, height = click.get_terminal_size()
+        width, height = shutil.get_terminal_size()
 
         err = maestral_err_list[0]
         err_title = cast(str, err["title"])
@@ -214,8 +216,8 @@ def check_for_fatal_errors(m: Union["MaestralProxy", "Maestral"]) -> bool:
 
 def convert_py_errors(func: Callable) -> Callable:
     """
-    Decorator that catches a MaestralApiError and prints it as a useful message to the
-    command line instead of printing the full stacktrace.
+    Decorator that catches a MaestralApiError and prints a formatted error message to
+    stdout before exiting. Calls ``sys.exit(1)`` after printing the error to stdout.
     """
 
     from .errors import MaestralApiError
@@ -225,7 +227,8 @@ def convert_py_errors(func: Callable) -> Callable:
         try:
             return func(*args, **kwargs)
         except MaestralApiError as exc:
-            raise cli.CliException(f"{exc.title}. {exc.message}")
+            cli.warn(f"{exc.title}. {exc.message}")
+            sys.exit(1)
 
     return wrapper
 
@@ -285,56 +288,78 @@ class DropboxPath(click.ParamType):
 
         return value
 
-    #
-    # def shell_complete(
-    #     self,
-    #     ctx: Optional[click.Context],
-    #     param: Optional[click.Parameter],
-    #     incomplete: str,
-    # ) -> List["CompletionItem"]:
-    #
-    #     from click.shell_completion import CompletionItem
-    #     from .utils import removeprefix
-    #
-    #     matches: List[str] = []
-    #
-    #     # check if we have been given an absolute path
-    #     incomplete = incomplete.lstrip("/")
-    #
-    #     # get the Maestral config for which to complete paths
-    #     try:
-    #         config_name = ctx.params["config_name"]
-    #     except (KeyError, AttributeError):
-    #         # attribute error occurs when ctx = None
-    #         config_name = "maestral"
-    #
-    #     # get all matching paths in our local Dropbox folder
-    #     # TODO: query from server if not too slow
-    #
-    #     config = MaestralConfig(config_name)
-    #     dropbox_dir = config.get("main", "path")
-    #     local_incomplete = osp.join(dropbox_dir, incomplete)
-    #     local_dirname = osp.dirname(local_incomplete)
-    #
-    #     if osp.isdir(local_dirname):
-    #
-    #         with os.scandir(local_dirname) as it:
-    #             for entry in it:
-    #                 if entry.path.startswith(local_incomplete):
-    #                     if entry.is_dir() and self.dir_okay:
-    #                         dbx_path = removeprefix(entry.path, dropbox_dir)
-    #                         matches.append(dbx_path + "/")
-    #                     elif entry.is_file() and self.file_okay:
-    #                         dbx_path = removeprefix(entry.path, dropbox_dir)
-    #                         matches.append(dbx_path)
-    #
-    #     # get all matching excluded items
-    #
-    #     for dbx_path in config.get("main", "excluded_items"):
-    #         if dbx_path.startswith("/" + incomplete):
-    #             matches.append(dbx_path)
-    #
-    #     return [CompletionItem(m.lstrip("/")) for m in matches]
+    def shell_complete(
+        self,
+        ctx: Optional[click.Context],
+        param: Optional[click.Parameter],
+        incomplete: str,
+    ) -> List["CompletionItem"]:
+
+        from click.shell_completion import CompletionItem
+        from .utils import removeprefix
+        from .config import MaestralConfig
+
+        matches: List[str] = []
+        completions: List[CompletionItem] = []
+
+        # check if we have been given an absolute path
+        absolute = incomplete.startswith("/")
+        incomplete = incomplete.lstrip("/")
+
+        # get the Maestral config for which to complete paths
+        config_name = ctx.params.get("config_name", "maestral") if ctx else "maestral"
+
+        # get all matching paths in our local Dropbox folder
+        # TODO: query from server if not too slow
+
+        config = MaestralConfig(config_name)
+        dropbox_dir = config.get("main", "path")
+        local_incomplete = osp.join(dropbox_dir, incomplete)
+        local_dirname = osp.dirname(local_incomplete)
+
+        try:
+            with os.scandir(local_dirname) as it:
+                for entry in it:
+                    if entry.path.startswith(local_incomplete):
+                        if self.file_okay and entry.is_file():
+                            dbx_path = removeprefix(entry.path, dropbox_dir)
+                            matches.append(dbx_path)
+                        if self.dir_okay and entry.is_dir():
+                            dbx_path = removeprefix(entry.path, dropbox_dir)
+                            matches.append(dbx_path)
+        except OSError:
+            pass
+
+        # get all matching excluded items
+
+        for dbx_path in config.get("main", "excluded_items"):
+            if dbx_path.startswith("/" + incomplete):
+                matches.append(dbx_path)
+
+        for match in matches:
+            if not absolute:
+                match = match.lstrip("/")
+            completions.append(CompletionItem(match))
+
+        return completions
+
+
+class ConfigKey(click.ParamType):
+    """A command line parameter representing a config key"""
+
+    name = "key"
+
+    def shell_complete(
+        self,
+        ctx: Optional[click.Context],
+        param: Optional[click.Parameter],
+        incomplete: str,
+    ) -> List["CompletionItem"]:
+
+        from click.shell_completion import CompletionItem
+        from .config.main import KEY_SECTION_MAP as KEYS
+
+        return [CompletionItem(key) for key in KEYS if key.startswith(incomplete)]
 
 
 class ConfigName(click.ParamType):
@@ -382,17 +407,18 @@ class ConfigName(click.ParamType):
                     f"Use 'maestral configs' to list all configurations."
                 )
 
-    #
-    # def shell_complete(
-    #     self,
-    #     ctx: Optional[click.Context],
-    #     param: Optional[click.Parameter],
-    #     incomplete: str,
-    # ) -> List["CompletionItem"]:
-    #
-    #     matches = [conf for conf in list_configs() if conf.startswith(incomplete)]
-    #     return [CompletionItem(m) for m in matches]
-    #
+    def shell_complete(
+        self,
+        ctx: Optional[click.Context],
+        param: Optional[click.Parameter],
+        incomplete: str,
+    ) -> List["CompletionItem"]:
+
+        from click.shell_completion import CompletionItem
+        from .config import list_configs
+
+        matches = [conf for conf in list_configs() if conf.startswith(incomplete)]
+        return [CompletionItem(m) for m in matches]
 
 
 # ======================================================================================
@@ -528,24 +554,21 @@ existing_config_option = click.option(
     "-v",
     is_flag=True,
     default=False,
-    help="Print log messages to stdout when started with '-f, --foreground' flag.",
+    help="Print log messages to stderr.",
 )
 @config_option
 @convert_py_errors
 def start(foreground: bool, verbose: bool, config_name: str) -> None:
 
-    # ---- run setup if necessary ------------------------------------------------------
-
-    # We run the setup in the current process. This avoids starting a subprocess despite
-    # running with the --foreground flag, prevents leaving a zombie process if the setup
-    # fails with an exception and does not confuse systemd.
-
+    import threading
     from .daemon import (
         MaestralProxy,
         start_maestral_daemon,
         start_maestral_daemon_process,
+        wait_for_startup,
         is_running,
         Start,
+        CommunicationError,
     )
 
     check_for_updates()
@@ -554,8 +577,68 @@ def start(foreground: bool, verbose: bool, config_name: str) -> None:
         click.echo("Daemon is already running.")
         return
 
-    if not foreground:
-        # start daemon process
+    @convert_py_errors
+    def startup_dialog():
+
+        try:
+            wait_for_startup(config_name)
+        except CommunicationError:
+            return
+
+        m = MaestralProxy(config_name)
+
+        if m.pending_link:
+            link_dialog(m)
+
+        if m.pending_dropbox_folder:
+            path = select_dbx_path_dialog(config_name, allow_merge=True)
+
+            while True:
+                try:
+                    m.create_dropbox_directory(path)
+                    break
+                except OSError:
+                    cli.warn(
+                        "Could not create folder. Please make sure that you have "
+                        "permissions to write to the selected location or choose a "
+                        "different location."
+                    )
+
+            include_all = cli.confirm("Would you like sync all folders?")
+
+            if not include_all:
+                # get all top-level Dropbox folders
+                cli.info("Loading...")
+                entries = m.list_folder("/", recursive=False)
+
+                names = [
+                    cast(str, e["name"])
+                    for e in entries
+                    if e["type"] == "FolderMetadata"
+                ]
+
+                choices = cli.select_multiple(
+                    "Choose which folders to include", options=names
+                )
+
+                excluded_paths = [
+                    f"/{name}"
+                    for index, name in enumerate(names)
+                    if index not in choices
+                ]
+
+                m.excluded_items = excluded_paths
+
+            cli.ok("Setup completed. Starting sync.")
+
+        m.start_sync()
+
+    t = threading.Thread(target=startup_dialog)
+    t.start()
+
+    if foreground:
+        start_maestral_daemon(config_name, log_to_stderr=verbose)
+    else:
         cli.echo("Starting Maestral...", nl=False)
 
         res = start_maestral_daemon_process(config_name)
@@ -569,54 +652,6 @@ def start(foreground: bool, verbose: bool, config_name: str) -> None:
             cli.echo("Please check logs for more information.")
             return
 
-    m = MaestralProxy(config_name, fallback=True)
-
-    if m.pending_link:  # this may raise KeyringAccessError
-        link_dialog(m)
-
-    if m.pending_dropbox_folder:
-        path = select_dbx_path_dialog(config_name, allow_merge=True)
-
-        while True:
-            try:
-                m.create_dropbox_directory(path)
-                break
-            except OSError:
-                cli.warn(
-                    "Could not create folder. Please make sure that you have "
-                    "permissions to write to the selected location or choose a "
-                    "different location."
-                )
-
-        include_all = cli.confirm("Would you like sync all folders?")
-
-        if not include_all:
-            # get all top-level Dropbox folders
-            cli.info("Loading...")
-            entries = m.list_folder("/", recursive=False)
-
-            names = [
-                cast(str, e["name"]) for e in entries if e["type"] == "FolderMetadata"
-            ]
-
-            choices = cli.select_multiple(
-                "Choose which folders to include", options=names
-            )
-
-            excluded_paths = [
-                f"/{name}" for index, name in enumerate(names) if index not in choices
-            ]
-
-            m.excluded_items = excluded_paths
-
-        cli.ok("Setup completed. Starting sync.")
-
-    if foreground:
-        del m
-        start_maestral_daemon(config_name, log_to_stdout=verbose, start_sync=True)
-    else:
-        m.start_sync()
-
 
 @main.command(section="Core Commands", help="Stop the sync daemon.")
 @existing_config_option
@@ -840,7 +875,7 @@ def sharelink_revoke(url: str, config_name: str) -> None:
     with MaestralProxy(config_name, fallback=True) as m:
         m.revoke_shared_link(url)
 
-    cli.echo("Revoked shared link.")
+    cli.confirm("Revoked shared link.")
 
 
 @sharelink.command(
@@ -983,12 +1018,14 @@ def activity(config_name: str) -> None:
                     height, width = screen.getmaxyx()
 
                     # create header
-                    lines = [f"Status: {m.status}, Sync errors: {len(m.sync_errors)}"]
-                    lines.append("")
+                    lines = [
+                        f"Status: {m.status}, Sync errors: {len(m.sync_errors)}",
+                        "",
+                    ]
 
                     # create table
-                    filenames = ["Path"]
-                    states = ["Status"]
+                    filenames = []
+                    states = []
                     col_len = 4
 
                     for event in m.get_activity(limit=height - 3):
@@ -1103,12 +1140,11 @@ def ls(long: bool, dropbox_path: str, include_deleted: bool, config_name: str) -
 
         cli.echo("Loading...\r", nl=False)
 
-        entries = m.list_folder(
+        entries_iter = m.list_folder_iterator(
             dropbox_path,
             recursive=False,
             include_deleted=include_deleted,
         )
-        entries.sort(key=lambda x: cast(str, x["name"]).lower())
 
         if long:
 
@@ -1129,37 +1165,39 @@ def ls(long: bool, dropbox_path: str, include_deleted: bool, config_name: str) -
                 ]
             )
 
-            for entry in entries:
+            for entries in entries_iter:
 
-                item_type = to_short_type[cast(str, entry["type"])]
-                name = cast(str, entry["name"])
-                path_lower = cast(str, entry["path_lower"])
+                for entry in entries:
 
-                text = "shared" if "sharing_info" in entry else "private"
-                color = "bright_black" if text == "private" else None
-                shared_field = cli.TextField(text, fg=color)
+                    item_type = to_short_type[cast(str, entry["type"])]
+                    name = cast(str, entry["name"])
+                    path_lower = cast(str, entry["path_lower"])
 
-                excluded_status = m.excluded_status(path_lower)
-                color = "green" if excluded_status == "included" else None
-                text = "✓" if excluded_status == "included" else excluded_status
-                excluded_field = cli.TextField(text, fg=color)
+                    text = "shared" if "sharing_info" in entry else "private"
+                    color = "bright_black" if text == "private" else None
+                    shared_field = cli.TextField(text, fg=color)
 
-                if "size" in entry:
-                    size = natural_size(cast(float, entry["size"]))
-                else:
-                    size = "-"
+                    excluded_status = m.excluded_status(path_lower)
+                    color = "green" if excluded_status == "included" else None
+                    text = "✓" if excluded_status == "included" else excluded_status
+                    excluded_field = cli.TextField(text, fg=color)
 
-                dt_field: cli.Field
+                    if "size" in entry:
+                        size = natural_size(cast(float, entry["size"]))
+                    else:
+                        size = "-"
 
-                if "client_modified" in entry:
-                    cm = cast(str, entry["client_modified"])
-                    dt_field = cli.DateField(_datetime_from_iso_str(cm))
-                else:
-                    dt_field = cli.TextField("-")
+                    dt_field: cli.Field
 
-                table.append(
-                    [name, item_type, size, shared_field, excluded_field, dt_field]
-                )
+                    if "client_modified" in entry:
+                        cm = cast(str, entry["client_modified"])
+                        dt_field = cli.DateField(_datetime_from_iso_str(cm))
+                    else:
+                        dt_field = cli.TextField("-")
+
+                    table.append(
+                        [name, item_type, size, shared_field, excluded_field, dt_field]
+                    )
 
             cli.echo(" " * 15)
             table.echo()
@@ -1169,11 +1207,12 @@ def ls(long: bool, dropbox_path: str, include_deleted: bool, config_name: str) -
 
             grid = cli.Grid()
 
-            for entry in entries:
-                name = cast(str, entry["name"])
-                color = "blue" if entry["type"] == "DeletedMetadata" else None
+            for entries in entries_iter:
+                for entry in entries:
+                    name = cast(str, entry["name"])
+                    color = "blue" if entry["type"] == "DeletedMetadata" else None
 
-                grid.append(cli.TextField(name, fg=color))
+                    grid.append(cli.TextField(name, fg=color))
 
             grid.echo()
 
@@ -1438,11 +1477,12 @@ Rebuilding may take several minutes, depending on the size of your Dropbox.
 def rebuild_index(yes: bool, config_name: str) -> None:
 
     import textwrap
+    import shutil
     from .daemon import MaestralProxy
 
     with MaestralProxy(config_name, fallback=True) as m:
 
-        width, height = click.get_terminal_size()
+        width, height = shutil.get_terminal_size()
 
         msg = textwrap.fill(
             "Rebuilding the index may take several minutes, depending on the size of "
@@ -1543,51 +1583,9 @@ def diff(
 
     from .daemon import MaestralProxy
 
-    def download_and_compare(
-        m: MaestralProxy, old_rev: str, new_rev: Optional[str] = None
-    ) -> None:
-        """
-        Download up to two revisions to a local temporary folder
-        and compare them with a 'diff'. Only text files are supported.
-        If an unknown file type was found, everything that doesn't match
-        'text/*', an error message gets printed.
-        """
-
-        diff = m.get_file_diff(old_rev, new_rev)
-
-        if len(diff) == 0:
-            click.echo("There are no changes between the two revisions.")
-            return
-
-        def color(ind: int, line: str) -> str:
-            """
-            Color diff lines.
-            Inspiration for colors was taken from the
-            well known command 'git diff'.
-            """
-
-            if ind < 2:
-                line = click.style(line, bold=True)
-            elif line.startswith("+"):
-                line = click.style(line, fg="green")
-            elif line.startswith("-"):
-                line = click.style(line, fg="red")
-            # Don't highlight these in the intro
-            elif line.startswith("@@ "):
-                line = click.style(line, fg="cyan")
-            return line
-
-        # Color the lines
-        if not no_color:
-            diff = [color(i, l) for i, l in enumerate(diff)]
-
-        # Enter pager if diff is too long
-        if len(diff) > 30 and not no_pager:
-            click.echo_via_pager("".join(diff))
-        else:
-            click.echo("".join(diff))
-
     with MaestralProxy(config_name, fallback=True) as m:
+
+        # Ask for user input if revs are not provided as CLI arguments.
         if len(rev) == 0:
             entries = m.list_revisions(dropbox_path, limit=limit)
 
@@ -1622,7 +1620,7 @@ def diff(
                 hint="(↓ to see more)" if len(comparable_versions) > 6 else "",
             )
 
-            old_rev = entries[index_new + index_base]["rev"]
+            old_rev = entries[index_new + index_base + 1]["rev"]
             new_rev = entries[index_base]["rev"]
         elif len(rev) == 1:
             old_rev = rev[0]
@@ -1634,10 +1632,46 @@ def diff(
             cli.warn("You can only compare two revisions at a time.")
             return
 
-        # '\r' will put the cursor to the beginning of the line
-        # so the next characters will overwrite it
+        # Download up to two revisions to a local temporary folder
+        # and compare them with a 'diff'. Only text files are supported.
+        # If an unknown file type was found, everything that doesn't match
+        # 'text/*', an error message gets printed.
+
         click.echo("Loading ...\r", nl=False)
-        download_and_compare(m, old_rev, new_rev)
+
+        diff_output = m.get_file_diff(old_rev, new_rev)
+
+        if len(diff_output) == 0:
+            click.echo("There are no changes between the two revisions.")
+            return
+
+        def color(ind: int, line: str) -> str:
+            """
+            Color diff lines.
+            Inspiration for colors was taken from the
+            well known command 'git diff'.
+            """
+
+            if ind < 2:
+                line = click.style(line, bold=True)
+            elif line.startswith("+"):
+                line = click.style(line, fg="green")
+            elif line.startswith("-"):
+                line = click.style(line, fg="red")
+            # Don't highlight these in the intro.
+            elif line.startswith("@@ "):
+                line = click.style(line, fg="cyan")
+            return line
+
+        # Color the lines.
+        if not no_color:
+            diff_output = [color(i, l) for i, l in enumerate(diff_output)]
+
+        # Enter pager if diff is too long
+        if len(diff_output) > 30 and not no_pager:
+            click.echo_via_pager("".join(diff_output))
+        else:
+            click.echo("".join(diff_output))
 
 
 @main.command(
@@ -1803,18 +1837,16 @@ def config():
 
 
 @config.command(name="get", help="Print the value of a given configuration key.")
-@click.argument("key")
+@click.argument("key", type=ConfigKey())
 @config_option
 def config_get(key: str, config_name: str) -> None:
 
     from .config import MaestralConfig
-    from .config.main import DEFAULTS_CONFIG
+    from .config.main import KEY_SECTION_MAP
     from .daemon import MaestralProxy, CommunicationError
 
     # Check if the config key exists in any section.
-    section = next(
-        iter(sec for sec, secdict in DEFAULTS_CONFIG.items() if key in secdict), None
-    )
+    section = KEY_SECTION_MAP.get(key, "")
 
     if not section:
         raise cli.CliException(f"'{key}' is not a valid configuration key.")
@@ -1837,7 +1869,7 @@ Values will be cast to the proper type, raising an error where this is not possi
 instance, setting a boolean config value to 1 will actually set it to True.
 """,
 )
-@click.argument("key")
+@click.argument("key", type=ConfigKey())
 @click.argument("value")
 @config_option
 @convert_py_errors
@@ -1845,20 +1877,15 @@ def config_set(key: str, value: str, config_name: str) -> None:
 
     import ast
     from .config import MaestralConfig
-    from .config.main import DEFAULTS_CONFIG
+    from .config.main import KEY_SECTION_MAP, DEFAULTS_CONFIG
     from .daemon import MaestralProxy, CommunicationError
 
-    section, defaults = next(
-        iter(
-            (sec, secdict) for sec, secdict in DEFAULTS_CONFIG.items() if key in secdict
-        ),
-        (None, None),
-    )
+    section = KEY_SECTION_MAP.get(key, "")
 
-    if not section or not defaults:
+    if not section:
         raise cli.CliException(f"'{key}' is not a valid configuration key.")
 
-    default_value = defaults[key]
+    default_value = DEFAULTS_CONFIG[section][key]
 
     if isinstance(default_value, str):
         py_value = value
@@ -1873,3 +1900,93 @@ def config_set(key: str, value: str, config_name: str) -> None:
             m.set_conf(section, key, py_value)
     except CommunicationError:
         MaestralConfig(config_name).set(section, key, py_value)
+
+
+@config.command(name="show", help="Show all config keys and values")
+@click.option("--no-pager", help="Don't use a pager for output.", is_flag=True)
+@config_option
+def config_show(no_pager: bool, config_name: str) -> None:
+
+    import io
+    from .config import MaestralConfig
+
+    conf = MaestralConfig(config_name)
+
+    with io.StringIO() as fp:
+        conf.write(fp)
+        if no_pager:
+            click.echo(fp.getvalue())
+        else:
+            click.echo_via_pager(fp.getvalue())
+
+
+@main.command(
+    section="Maintenance",
+    help="""
+Generate completion script for your shell.
+
+This command can generate shell completion scripts for bash, zsh or fish. Follow the
+instructions below for your shell to load the resulting script. The exact config file
+locations might vary based on your system. Make sure to restart your
+shell before testing whether completions are working.
+
+### bash
+
+You can enable shell completion for all users by generating and saving the script as
+follows:
+
+\b
+    maestral completion bash > /usr/share/bash-completion/completions/maestral
+
+To enable shell completion for the current user only, save the script in a location of
+your choice, for example `~/.local/completions/maestral`, and source it in `~/.bashrc`
+by adding the line:
+
+\b
+    . ~/.local/completions/maestral
+
+### zsh
+
+Generate a `_maestral` completion script and put it somewhere in your `$fpath`. For
+example:
+
+\b
+    maestral completion zsh > /usr/local/share/zsh/site-functions/_maestral
+
+You can also save the completion script in a location of your choice and source it
+in `~/.zshrc`. Ensure that the following is present in your `~/.zshrc`:
+
+\b
+    autoload -Uz compinit && compinit
+
+### fish
+
+Generate and save a `maestral.fish` completion script as follows. For all users:
+
+\b
+    maestral completion fish > /usr/share/fish/vendor_completions.d/maestral.fish
+
+For the current user only:
+
+\b
+    maestral completion fish > ~/.config/fish/completions/maestral.fish
+
+""",
+)
+@click.argument("shell", type=click.Choice(["bash", "zsh", "fish"]))
+def completion(shell: str) -> None:
+
+    from click.shell_completion import get_completion_class
+
+    comp_cls = get_completion_class(shell)
+
+    if comp_cls is None:
+        cli.warn(f"{shell} shell is currently not supported")
+        return
+
+    comp = comp_cls(main, {}, "maestral", "_MAESTRAL_COMPLETE")
+
+    try:
+        click.echo(comp.source())
+    except RuntimeError as exc:
+        cli.warn(exc.args[0])
