@@ -38,6 +38,7 @@ from dropbox import (  # type: ignore
     auth,
     oauth,
 )
+from dropbox.stone_validators import ValidationError
 
 # local imports
 from . import __version__
@@ -75,7 +76,7 @@ from .constants import DROPBOX_APP_KEY
 from .utils import natural_size, chunks, clamp
 
 if TYPE_CHECKING:
-    from .sync import SyncEvent
+    from .database import SyncEvent
 
 
 __all__ = [
@@ -87,8 +88,6 @@ __all__ = [
 ]
 
 
-logger = logging.getLogger(__name__)
-
 # type definitions
 LocalError = Union[MaestralApiError, OSError]
 WriteErrorType = Type[
@@ -123,7 +122,6 @@ SessionLookupErrorType = Type[
 PaginationResultType = Union[sharing.ListSharedLinksResult, files.ListFolderResult]
 FT = TypeVar("FT", bound=Callable[..., Any])
 
-# create single requests session for all clients
 _major_minor_version = ".".join(__version__.split(".")[:2])
 USER_AGENT = f"Maestral/v{_major_minor_version}"
 
@@ -143,15 +141,18 @@ def convert_api_errors(
 ) -> Iterator[None]:
     """
     A context manager that catches and re-raises instances of :class:`OSError` and
-    :class:`exceptions.DropboxException` as :class:`errors.MaestralApiError` or
-    :class:`ConnectionError`.
+    :class:`dropbox.exceptions.DropboxException` as
+    :class:`maestral.errors.MaestralApiError` or :class:`ConnectionError`.
+
+    :param dbx_path: Dropbox path associated with the error.
+    :param local_path: Local path associated with the error.
     """
 
     try:
         yield
-    except exceptions.DropboxException as exc:
+    except (exceptions.DropboxException, ValidationError, requests.HTTPError) as exc:
         raise dropbox_to_maestral_error(exc, dbx_path, local_path)
-    # catch connection errors first, they may inherit from OSError
+    # Catch connection errors first, they may inherit from OSError.
     except CONNECTION_ERRORS:
         raise DropboxConnectionError(
             "Cannot connect to Dropbox",
@@ -159,7 +160,7 @@ def convert_api_errors(
         )
     except OSError as exc:
         if exc.errno == errno.EPROTOTYPE:
-            # Can occur on macOS, see https://bugs.python.org/issue33450
+            # Can occur on macOS, see https://bugs.python.org/issue33450.
             raise DropboxConnectionError(
                 "Cannot connect to Dropbox",
                 "Please check you internet connection and try again later.",
@@ -175,15 +176,24 @@ class DropboxClient:
     creating, moving, modifying and deleting files and folders on Dropbox and
     downloading files from Dropbox.
 
-    All Dropbox SDK exceptions and :class:`OSError` instances if related to accessing or
-    saving local files will be caught and reraised as a
-    :class:`errors.MaestralApiError`. Connection errors from requests will be caught and
-    reraised as :class:`ConnectionError`.
+    All Dropbox SDK exceptions, OSErrors from the local file system API and connection
+    errors will be caught and reraised as a subclass of
+    :class:`maestral.errors.MaestralApiError`.
+
+    This class can be used as a context manager to clean up any network resources from
+    the API requests.
+
+    :Example:
+
+        >>> from maestral.client import DropboxClient
+        >>> with DropboxClient("maestral") as client:
+        ...     res = client.list_folder("/")
+        >>> print(res.entries)
 
     :param config_name: Name of config file and state file to use.
     :param timeout: Timeout for individual requests. Defaults to 100 sec if not given.
     :param session: Optional requests session to use. If not given, a new session will
-        be created with :function:`dropbox.create_session`.
+        be created with :func:`dropbox.dropbox_client.create_session`.
     """
 
     SDK_VERSION: str = "2.0"
@@ -200,6 +210,8 @@ class DropboxClient:
         self.config_name = config_name
         self.auth = OAuth2Session(config_name)
 
+        self._logger = logging.getLogger(__name__)
+
         self._timeout = timeout
         self._session = session or create_session()
         self._backoff_until = 0
@@ -210,7 +222,7 @@ class DropboxClient:
 
     @property
     def dbx(self) -> Dropbox:
-        """The actual Python Dropbox SDK"""
+        """The underlying Dropbox SDK instance."""
         if not self.linked:
             raise NotLinkedError(
                 "No auth token set", "Please link a Dropbox account first."
@@ -347,10 +359,11 @@ class DropboxClient:
     ) -> "DropboxClient":
         """
         Creates a new copy of the Dropbox client with the same defaults unless modified
-        by arguments to clone().
-
-        See constructor for original parameter descriptions.
+        by arguments to :meth:`clone`.
 
+        :param config_name: Name of config file and state file to use.
+        :param timeout: Timeout for individual requests.
+        :param session: Requests session to use.
         :returns: A new instance of DropboxClient.
         """
 
@@ -370,8 +383,7 @@ class DropboxClient:
     def clone_with_new_session(self) -> "DropboxClient":
         """
         Creates a new copy of the Dropbox client with the same defaults but a new
-        requests session. Use :meth:`close()` to clean up any resources from the network
-        session when done using the copy.
+        requests session.
 
         :returns: A new instance of DropboxClient.
         """
@@ -395,7 +407,7 @@ class DropboxClient:
                 res = self.dbx.users_get_current_account()
 
         if not dbid:
-            # save our own account info to config
+            # Save our own account info to config.
             if res.account_type.is_basic():
                 account_type = "basic"
             elif res.account_type.is_business():
@@ -419,7 +431,7 @@ class DropboxClient:
         with convert_api_errors():
             res = self.dbx.users_get_space_usage()
 
-        # query space usage type
+        # Query space usage type.
         if res.allocation.is_team():
             usage_type = "team"
         elif res.allocation.is_individual():
@@ -427,7 +439,7 @@ class DropboxClient:
         else:
             usage_type = ""
 
-        # generate space usage string
+        # Generate space usage string.
         if res.allocation.is_team():
             used = res.allocation.get_team().used
             allocated = res.allocation.get_team().allocated
@@ -438,7 +450,7 @@ class DropboxClient:
         percent = used / allocated
         space_usage = f"{percent:.1%} of {natural_size(allocated)} used"
 
-        # save results to config
+        # Save results to config.
         self._state.set("account", "usage", space_usage)
         self._state.set("account", "usage_type", usage_type)
 
@@ -470,7 +482,7 @@ class DropboxClient:
         :param dbx_path: Path to file on Dropbox.
         :param mode: Must be 'path' or 'id'. If 'id', specify the Dropbox file ID
             instead of the file path to get revisions across move and rename events.
-        :param limit: Maximum number of revisions to list. Defaults to 10.
+        :param limit: Maximum number of revisions to list.
         :returns: File revision history.
         """
 
@@ -499,13 +511,13 @@ class DropboxClient:
         **kwargs,
     ) -> files.FileMetadata:
         """
-        Downloads file from Dropbox to our local folder.
+        Downloads a file from Dropbox to given local path.
 
         :param dbx_path: Path to file on Dropbox or rev number.
         :param local_path: Path to local download destination.
         :param sync_event: If given, the sync event will be updated with the number of
             downloaded bytes.
-        :param kwargs: Keyword arguments for Dropbox SDK files_download.
+        :param kwargs: Keyword arguments for the Dropbox API files_download endpoint.
         :returns: Metadata of downloaded item.
         """
 
@@ -528,13 +540,13 @@ class DropboxClient:
                         if sync_event:
                             sync_event.completed = f.tell()
 
-        # dropbox SDK provides naive datetime in UTC
+        # Dropbox SDK provides naive datetime in UTC.
         client_mod = md.client_modified.replace(tzinfo=timezone.utc)
         server_mod = md.server_modified.replace(tzinfo=timezone.utc)
 
-        # enforce client_modified < server_modified
+        # Enforce client_modified < server_modified.
         timestamp = min(client_mod.timestamp(), server_mod.timestamp(), time.time())
-        # set mtime of downloaded file
+        # Set mtime of downloaded file.
         os.utime(local_path, (time.time(), timestamp))
 
         return md
@@ -566,7 +578,7 @@ class DropboxClient:
 
             size = osp.getsize(local_path)
 
-            # dropbox SDK takes naive datetime in UTC
+            # Dropbox SDK takes naive datetime in UTC/
             mtime = osp.getmtime(local_path)
             mtime_dt = datetime.utcfromtimestamp(mtime)
 
@@ -601,7 +613,7 @@ class DropboxClient:
                         try:
 
                             if size - f.tell() <= chunk_size:
-                                # Wrap up upload session and return metadata.
+                                # Finish upload session and return metadata.
                                 data = f.read(chunk_size)
                                 md = self.dbx.files_upload_session_finish(
                                     data, cursor, commit
@@ -633,7 +645,7 @@ class DropboxClient:
                                 raise exc
 
                             if session_lookup_error.is_incorrect_offset():
-                                # reset position in file
+                                # Reset position in file.
                                 offset = (
                                     session_lookup_error.get_incorrect_offset().correct_offset
                                 )
@@ -647,7 +659,7 @@ class DropboxClient:
         Removes a file / folder from Dropbox.
 
         :param dbx_path: Path to file on Dropbox.
-        :param kwargs: Keyword arguments for Dropbox SDK files_delete_v2.
+        :param kwargs: Keyword arguments for the Dropbox API files_delete_v2 endpoint.
         :returns: Metadata of deleted item.
         """
 
@@ -659,15 +671,15 @@ class DropboxClient:
         self, entries: List[Tuple[str, str]], batch_size: int = 900
     ) -> List[Union[files.Metadata, MaestralApiError]]:
         """
-        Delete multiple items on Dropbox in a batch job.
+        Deletes multiple items on Dropbox in a batch job.
 
         :param entries: List of Dropbox paths and "rev"s to delete. If a "rev" is not
             None, the file will only be deleted if it matches the rev on Dropbox. This
             is not supported when deleting a folder.
         :param batch_size: Number of items to delete in each batch. Dropbox allows
             batches of up to 1,000 items. Larger values will be capped automatically.
-        :returns: List of Metadata for deleted items or :class:`errors.SyncError` for
-            failures. Results will be in the same order as the original input.
+        :returns: List of Metadata for deleted items or SyncErrors for failures. Results
+            will be in the same order as the original input.
         """
 
         batch_size = clamp(batch_size, 1, 1000)
@@ -675,7 +687,7 @@ class DropboxClient:
         res_entries = []
         result_list = []
 
-        # up two ~ 1,000 entries allowed per batch according to
+        # Up two ~ 1,000 entries allowed per batch:
         # https://www.dropbox.com/developers/reference/data-ingress-guide
         for chunk in chunks(entries, n=batch_size):
 
@@ -738,7 +750,7 @@ class DropboxClient:
 
         :param dbx_path: Path to file/folder on Dropbox.
         :param new_path: New path on Dropbox to move to.
-        :param kwargs: Keyword arguments for Dropbox SDK files_move_v2.
+        :param kwargs: Keyword arguments for the Dropbox API files_move_v2 endpoint.
         :returns: Metadata of moved item.
         """
 
@@ -757,7 +769,8 @@ class DropboxClient:
         Creates a folder on Dropbox.
 
         :param dbx_path: Path of Dropbox folder.
-        :param kwargs: Keyword arguments for Dropbox SDK files_create_folder_v2.
+        :param kwargs: Keyword arguments for the Dropbox API files_create_folder_v2
+            endpoint.
         :returns: Metadata of created folder.
         """
 
@@ -774,7 +787,8 @@ class DropboxClient:
         :param dbx_paths: List of dropbox folder paths.
         :param batch_size: Number of folders to create in each batch. Dropbox allows
             batches of up to 1,000 folders. Larger values will be capped automatically.
-        :param kwargs: Keyword arguments for Dropbox SDK files_create_folder_batch.
+        :param kwargs: Keyword arguments for the Dropbox API files/create_folder_batch
+            endpoint.
         :returns: List of Metadata for created folders or SyncError for failures.
             Entries will be in the same order as given paths.
         """
@@ -785,7 +799,7 @@ class DropboxClient:
 
         with convert_api_errors():
 
-            # up two ~ 1,000 entries allowed per batch according to
+            # Up two ~ 1,000 entries allowed per batch:
             # https://www.dropbox.com/developers/reference/data-ingress-guide
             for chunk in chunks(dbx_paths, n=batch_size):
                 res = self.dbx.files_create_folder_batch(chunk, **kwargs)
@@ -840,7 +854,8 @@ class DropboxClient:
         :param dbx_path: Path of folder on Dropbox.
         :param include_non_downloadable_files: If ``True``, files that cannot be
             downloaded (at the moment only G-suite files on Dropbox) will be included.
-        :param kwargs: Other keyword arguments for Dropbox SDK files_list_folder.
+        :param kwargs: Additional keyword arguments for Dropbox API
+            files/list_folder/get_latest_cursor endpoint.
         :returns: The latest cursor representing a state of a folder and its subfolders.
         """
 
@@ -866,7 +881,7 @@ class DropboxClient:
         """
         Lists the contents of a folder on Dropbox. Similar to
         :meth:`list_folder_iterator` but returns all entries in a single
-        :class:`files.ListFolderResult` instance.
+        :class:`dropbox.files.ListFolderResult` instance.
 
         :param dbx_path: Path of folder on Dropbox.
         :param max_retries_on_timeout: Number of times to try again if Dropbox servers
@@ -874,7 +889,8 @@ class DropboxClient:
             large Dropbox folders.
         :param include_non_downloadable_files: If ``True``, files that cannot be
             downloaded (at the moment only G-suite files on Dropbox) will be included.
-        :param kwargs: Other keyword arguments for Dropbox SDK files_list_folder.
+        :param kwargs: Additional keyword arguments for Dropbox API files/list_folder
+            endpoint.
         :returns: Content of given folder.
         """
 
@@ -896,9 +912,9 @@ class DropboxClient:
     ) -> Iterator[files.ListFolderResult]:
         """
         Lists the contents of a folder on Dropbox. Returns an iterator yielding
-        :class:`files.ListFolderResult` instances. The number of entries returned in
-        each iteration corresponds to the number of entries returned by a single Dropbox
-        API call and will be typically around 500.
+        :class:`dropbox.files.ListFolderResult` instances. The number of entries
+        returned in each iteration corresponds to the number of entries returned by a
+        single Dropbox API call and will be typically around 500.
 
         :param dbx_path: Path of folder on Dropbox.
         :param max_retries_on_timeout: Number of times to try again if Dropbox servers
@@ -906,7 +922,8 @@ class DropboxClient:
             large Dropbox folders.
         :param include_non_downloadable_files: If ``True``, files that cannot be
             downloaded (at the moment only G-suite files on Dropbox) will be included.
-        :param kwargs: Other keyword arguments for Dropbox SDK files_list_folder.
+        :param kwargs: Additional keyword arguments for the Dropbox API
+            files/list_folder endpoint.
         :returns: Iterator over content of given folder.
         """
 
@@ -944,23 +961,24 @@ class DropboxClient:
         starting the Dropbox client and periodically to get the latest updates.
 
         :param last_cursor: Last to cursor to compare for changes.
-        :param timeout: Seconds to wait until timeout. Must be between 30 and 480.
+        :param timeout: Seconds to wait until timeout. Must be between 30 and 480. The
+            Dropbox API will add a random jitter of up to 60 sec to this value.
         :returns: ``True`` if changes are available, ``False`` otherwise.
         """
 
         if not 30 <= timeout <= 480:
             raise ValueError("Timeout must be in range [30, 480]")
 
-        # honour last request to back off
+        # Honour last request to back off.
         time_to_backoff = max(self._backoff_until - time.time(), 0)
         time.sleep(time_to_backoff)
 
         with convert_api_errors():
             res = self.dbx.files_list_folder_longpoll(last_cursor, timeout=timeout)
 
-        # keep track of last longpoll, back off if requested by SDK
+        # Keep track of last longpoll, back off if requested by API.
         if res.backoff:
-            logger.debug("Backoff requested for %s sec", res.backoff)
+            self._logger.debug("Backoff requested for %s sec", res.backoff)
             self._backoff_until = time.time() + res.backoff + 5.0
         else:
             self._backoff_until = 0
@@ -971,8 +989,8 @@ class DropboxClient:
         """
         Lists changes to remote Dropbox since ``last_cursor``. Same as
         :meth:`list_remote_changes_iterator` but fetches all changes first and returns
-        a single :class:`files.ListFolderResult`. This may be useful if you want to
-        fetch all changes before starting to process them.
+        a single :class:`dropbox.files.ListFolderResult`. This may be useful if you want
+        to fetch all changes in advance before starting to process them.
 
         :param last_cursor: Last to cursor to compare for changes.
         :returns: Remote changes since given cursor.
@@ -986,9 +1004,9 @@ class DropboxClient:
     ) -> Iterator[files.ListFolderResult]:
         """
         Lists changes to the remote Dropbox since ``last_cursor``. Returns an iterator
-        yielding :class:`files.ListFolderResult` instances. The number of entries
-        returned in each iteration corresponds to the number of entries returned by a
-        single Dropbox API call and will be typically around 500.
+        yielding :class:`dropbox.files.ListFolderResult` instances. The number of
+        entries returned in each iteration corresponds to the number of entries returned
+        by a single Dropbox API call and will be typically around 500.
 
         Call this after :meth:`wait_for_remote_changes` returns ``True``.
 
@@ -1029,8 +1047,8 @@ class DropboxClient:
             is set to password protected and will be ignored otherwise
         :param expires: Expiry time for shared link. Only available for Professional and
             Business accounts.
-        :param kwargs: Additional keyword arguments for the
-            :class:`dropbox.sharing.SharedLinkSettings`.
+        :param kwargs: Additional keyword arguments to create the
+            :class:`dropbox.sharing.SharedLinkSettings` instance.
         :returns: Metadata for shared link.
         """
 
@@ -1043,7 +1061,7 @@ class DropboxClient:
         if not visibility.is_password():
             password = None
 
-        # convert timestamp to utc time if not naive
+        # Convert timestamp to utc time if not naive.
         if expires is not None:
             has_timezone = expires.tzinfo and expires.tzinfo.utcoffset(expires)
             if has_timezone:
@@ -1100,7 +1118,7 @@ class DropboxClient:
     ) -> PaginationResultType:
         """
         Flattens a list of Dropbox API results from a pagination to a single result with
-        the cursor of the last entry in the list.
+        the cursor of the last result in the list.
 
         :param results: List of :results to flatten.
         :param attribute_name: Name of attribute to flatten.
@@ -1131,21 +1149,13 @@ def os_to_maestral_error(
     exc: OSError, dbx_path: Optional[str] = None, local_path: Optional[str] = None
 ) -> LocalError:
     """
-    Converts a :class:`OSError` to a :class:`MaestralApiError` and tries to add a
-    reasonably informative error title and message.
+    Converts a :class:`OSError` to a :class:`maestral.errors.MaestralApiError` and tries
+    to add a reasonably informative error title and message.
 
-    .. note::
-        The following exception types should not typically be raised during syncing:
-
-        * InterruptedError: The client will automatically retry on interrupted
-          connections.
-        * NotADirectoryError: If raised, this is likely a bug.
-        * IsADirectoryError: If raised, this is likely a bug.
-
-    :param exc: Python Exception.
-    :param dbx_path: Dropbox path of file which triggered the error.
-    :param local_path: Local path of file which triggered the error.
-    :returns: :class:`MaestralApiError` instance or :class:`OSError` instance.
+    :param exc: Original OSError.
+    :param dbx_path: Dropbox path associated with the error.
+    :param local_path: Local path associated with the error.
+    :returns: Converted exception.
     """
 
     title = "Could not sync file or folder"
@@ -1198,6 +1208,8 @@ def os_to_maestral_error(
     else:
         return exc
 
+    local_path = local_path or exc.filename
+
     maestral_exc = err_cls(title, text, dbx_path=dbx_path, local_path=local_path)
     maestral_exc.__cause__ = exc
 
@@ -1205,18 +1217,18 @@ def os_to_maestral_error(
 
 
 def dropbox_to_maestral_error(
-    exc: exceptions.DropboxException,
+    exc: Union[exceptions.DropboxException, ValidationError, requests.HTTPError],
     dbx_path: Optional[str] = None,
     local_path: Optional[str] = None,
 ) -> MaestralApiError:
     """
-    Converts a Dropbox SDK exception to a :class:`MaestralApiError` and tries to add a
-    reasonably informative error title and message.
+    Converts a Dropbox SDK exception to a :class:`maestral.errors.MaestralApiError` and
+    tries to add a reasonably informative error title and message.
 
-    :param exc: :class:`dropbox.exceptions.DropboxException` instance.
-    :param dbx_path: Dropbox path of file which triggered the error.
-    :param local_path: Local path of file which triggered the error.
-    :returns: :class:`MaestralApiError` instance.
+    :param exc: Dropbox SDK exception..
+    :param dbx_path: Dropbox path associated with the error.
+    :param local_path: Local path associated with the error.
+    :returns: Converted exception.
     """
 
     title = "An unexpected error occurred"
@@ -1316,16 +1328,16 @@ def dropbox_to_maestral_error(
                 write_error = error.get_path().reason  # returns UploadWriteFailed
                 text, err_cls = _get_write_error_msg(write_error)
             elif error.is_properties_error():
-                # this is a programming error in maestral
+                # Occurs only for programming error in maestral.
                 text = "Invalid property group provided."
 
         elif isinstance(error, files.UploadSessionStartError):
             title = "Could not upload file"
             if error.is_concurrent_session_close_not_allowed():
-                # this is a programming error in maestral
+                # Occurs only for programming error in maestral.
                 text = "Can not start a closed concurrent upload session."
             elif error.is_concurrent_session_data_not_allowed():
-                # this is a programming error in maestral
+                # Occurs only for programming error in maestral.
                 text = "Uploading data not allowed when starting concurrent upload session."
 
         elif isinstance(error, files.UploadSessionFinishError):
@@ -1337,7 +1349,7 @@ def dropbox_to_maestral_error(
                 write_error = error.get_path()
                 text, err_cls = _get_write_error_msg(write_error)
             elif error.is_properties_error():
-                # this is a programming error in maestral
+                # Occurs only for programming error in maestral.
                 text = "Invalid property group provided."
             elif error.is_too_many_write_operations():
                 text = (
@@ -1530,9 +1542,23 @@ def dropbox_to_maestral_error(
 
     # ---- OAuth2 flow errors ----------------------------------------------------------
     elif isinstance(exc, requests.HTTPError):
-        err_cls = DropboxAuthError
-        title = "Authentication failed"
-        text = "Please make sure that you entered the correct authentication code."
+        # HTTPErrors are converted to a DropboxException by the SDK unless they occur
+        # when refreshing the access token. We therefore handle those manually.
+        # See https://github.com/dropbox/dropbox-sdk-python/issues/360
+        # and https://github.com/SamSchott/maestral/issues/388.
+
+        if exc.request is not None and exc.request.status_code >= 500:
+            err_cls = DropboxServerError
+            title = "Dropbox server error"
+            text = (
+                "Something went wrong on Dropbox’s end. Please check on "
+                "status.dropbox.com if their services are up and running and try again "
+                "later."
+            )
+        else:
+            err_cls = DropboxAuthError
+            title = "Authentication failed"
+            text = "Please make sure that you entered the correct authentication code."
 
     elif isinstance(exc, oauth.BadStateException):
         err_cls = DropboxAuthError
@@ -1546,7 +1572,7 @@ def dropbox_to_maestral_error(
 
     # ---- Bad input errors ------------------------------------------------------------
     # should only occur due to user input from console scripts
-    elif isinstance(exc, exceptions.BadInputError):
+    elif isinstance(exc, (exceptions.BadInputError, ValidationError)):
         err_cls = BadInputError
         title = "Bad input to API call"
         text = exc.message
@@ -1667,14 +1693,14 @@ def _get_session_lookup_error_msg(
     err_cls = SyncError
 
     if session_lookup_error.is_closed():
-        # happens when trying to append data to a closed session
-        # this is caused by internal Maestral errors
+        # Occurs when trying to append data to a closed session.
+        # This is caused by internal Maestral errors.
         pass
     elif session_lookup_error.is_incorrect_offset():
         text = "A network error occurred during the upload session."
     elif session_lookup_error.is_not_closed():
-        # happens when trying to finish an open session
-        # this is caused by internal Maestral errors
+        # Occurs when trying to finish an open session.
+        # This is caused by internal Maestral errors.
         pass
     elif session_lookup_error.is_not_found():
         text = (
