@@ -14,12 +14,11 @@ import subprocess
 import threading
 import fcntl
 import struct
-import tempfile
-import logging
 import warnings
 import argparse
 import ast
 import re
+from pprint import pformat
 from shlex import quote
 from typing import Optional, Any, Union, Tuple, Dict, Iterable, Type, TYPE_CHECKING
 from types import TracebackType
@@ -33,7 +32,9 @@ from fasteners import InterProcessLock  # type: ignore
 
 # local imports
 from .errors import SYNC_ERRORS, GENERAL_ERRORS, MaestralApiError
+from .utils import exc_info_tuple
 from .utils.appdirs import get_runtime_path
+from .constants import IS_MACOS, ENV
 
 
 if TYPE_CHECKING:
@@ -48,6 +49,7 @@ __all__ = [
     "get_maestral_pid",
     "sockpath_for_config",
     "lockpath_for_config",
+    "wait_for_startup",
     "is_running",
     "freeze_support",
     "start_maestral_daemon",
@@ -58,9 +60,6 @@ __all__ = [
 ]
 
 
-logger = logging.getLogger(__name__)
-
-
 # systemd environment
 INVOCATION_ID = os.getenv("INVOCATION_ID")
 NOTIFY_SOCKET = os.getenv("NOTIFY_SOCKET")
@@ -82,7 +81,7 @@ def freeze_support() -> None:
     detected and do nothing otherwise.
     """
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument("-c")
     parsed_args, _ = parser.parse_known_args()
 
@@ -120,7 +119,7 @@ class Start(enum.Enum):
 
 def serpent_deserialize_api_error(class_name: str, d: dict) -> MaestralApiError:
     """
-    Deserializes a :class:`errors.MaestralApiError`.
+    Deserializes a MaestralApiError.
 
     :param class_name: Name of class to deserialize.
     :param d: Dictionary of serialized class.
@@ -160,10 +159,7 @@ def _get_lockdata() -> Tuple[bytes, str, int]:
     else:
         start_len = "qq"
 
-    if (
-        sys.platform.startswith(("netbsd", "freebsd", "openbsd"))
-        or sys.platform == "darwin"
-    ):
+    if sys.platform.startswith(("netbsd", "freebsd", "openbsd")) or IS_MACOS:
         if struct.calcsize("l") == 8:
             off_t = "l"
             pid_t = "i"
@@ -198,41 +194,40 @@ class Lock:
     This internally uses :class:`fasteners.InterProcessLock` but provides non-blocking
     acquire. It also guarantees thread-safety when using the :meth:`singleton` class
     method to create / retrieve a lock instance.
+
+    :param path: Path of the lock file to use / create.
     """
 
     _instances: Dict[str, "Lock"] = {}
     _singleton_lock = threading.Lock()
 
     @classmethod
-    def singleton(cls, name: str, lock_path: Optional[str] = None) -> "Lock":
+    def singleton(cls, path: str) -> "Lock":
         """
         Retrieve an existing lock object with a given 'name' or create a new one. Use
         this method for thread-safe locks.
 
-        :param name: Name of lock file.
-        :param lock_path: Directory for lock files. Defaults to the temporary directory
-            returned by :func:`tempfile.gettempdir()` if not given.
+        :param path: Path of the lock file to use / create.
         """
 
         with cls._singleton_lock:
             try:
-                instance = cls._instances[name]
+                instance = cls._instances[path]
             except KeyError:
-                instance = cls(name, lock_path)
-                cls._instances[name] = instance
+                instance = cls(path)
+                cls._instances[path] = instance
 
             return instance
 
-    def __init__(self, name: str, lock_path: Optional[str] = None) -> None:
+    def __init__(self, path: str) -> None:
 
-        self.name = name
-        dirname = lock_path or tempfile.gettempdir()
-        lock_path = os.path.join(dirname, name)
+        self.path = path
 
         self._internal_lock = threading.Semaphore()
-        self._external_lock = InterProcessLock(lock_path)
+        self._external_lock = InterProcessLock(self.path)
 
         self._lock = threading.RLock()
+        self._acquired = False  # tracks if we acquired the lock
 
     def acquire(self) -> bool:
         """
@@ -255,6 +250,7 @@ class Lock:
             else:
 
                 if locked_external:
+                    self._acquired = True
                     return True
                 else:
                     self._internal_lock.release()
@@ -263,11 +259,22 @@ class Lock:
     def release(self) -> None:
         """Release the previously acquired lock."""
         with self._lock:
+
+            if not self._acquired:
+                raise RuntimeError(
+                    "Cannot release a lock, it was acquired by a different process"
+                )
+
             self._external_lock.release()
             self._internal_lock.release()
+            self._acquired = False
 
     def locked(self) -> bool:
-        """Checks if the lock is currently held by any thread or process."""
+        """
+        Checks if the lock is currently held by any thread or process.
+
+        :returns: Whether the lock is acquired.
+        """
         with self._lock:
             gotten = self.acquire()
             if gotten:
@@ -285,11 +292,11 @@ class Lock:
 
         with self._lock:
 
-            if self._external_lock.acquired:
+            if self._acquired:
                 return os.getpid()
 
             try:
-                # don't close again in case we are the locking process
+                # Don't close again in case we are the locking process.
                 self._external_lock._do_open()
                 lockdata, fmt, pid_index = _get_lockdata()
                 lockdata = fcntl.fcntl(
@@ -321,33 +328,42 @@ def _send_term(pid: int) -> None:
 def maestral_lock(config_name: str) -> Lock:
     """
     Returns an inter-process and inter-thread lock for Maestral. This is a wrapper
-    around :class:`Lock` which fills out the appropriate lockfile name and directory for
-    the given config name.
+    around :class:`Lock` which fills out the appropriate lockfile path for he given
+    config name.
+
+    :param config_name: The name of the Maestral configuration.
+    :returns: Lock instance for the config name
     """
     name = f"{config_name}.lock"
     path = get_runtime_path("maestral")
-    return Lock.singleton(name, path)
+    return Lock.singleton(os.path.join(path, name))
 
 
 def sockpath_for_config(config_name: str) -> str:
     """
     Returns the unix socket location to be used for the config. This should default to
     the apps runtime directory + 'CONFIG_NAME.sock'.
+
+    :param config_name: The name of the Maestral configuration.
+    :returns: Socket path.
     """
     return get_runtime_path("maestral", f"{config_name}.sock")
 
 
 def lockpath_for_config(config_name: str) -> str:
     """
-    Returns the lock file location to be used for the config. This should default to
-    the apps runtime directory + 'CONFIG_NAME.lock'.
+    Returns the lock file location to be used for the config. This will be the apps
+    runtime directory + 'CONFIG_NAME.lock'.
+
+    :param config_name: The name of the Maestral configuration.
+    :returns: Path of lock file to use.
     """
     return get_runtime_path("maestral", f"{config_name}.lock")
 
 
 def get_maestral_pid(config_name: str) -> Optional[int]:
     """
-    Returns Maestral's PID if the daemon is running, ``None`` otherwise.
+    Returns the PID of the daemon if it is running, ``None`` otherwise.
 
     :param config_name: The name of the Maestral configuration.
     :returns: The daemon's PID.
@@ -367,7 +383,7 @@ def is_running(config_name: str) -> bool:
     return maestral_lock(config_name).locked()
 
 
-def _wait_for_startup(config_name: str, timeout: float) -> None:
+def wait_for_startup(config_name: str, timeout: float = 5) -> None:
     """
     Waits until we can communicate with the maestral daemon for ``config_name``.
 
@@ -399,90 +415,74 @@ def _wait_for_startup(config_name: str, timeout: float) -> None:
 
 
 def start_maestral_daemon(
-    config_name: str = "maestral", log_to_stdout: bool = False, start_sync: bool = False
+    config_name: str = "maestral", log_to_stderr: bool = False, start_sync: bool = False
 ) -> None:
     """
-    Starts the Maestral daemon with event loop in the current thread. Startup is race
-    free: there will never be two daemons running for the same config.
+    Starts the Maestral daemon with event loop in the current thread.
 
-    Wraps :class:`main.Maestral` as Pyro daemon object, creates a new instance and
-    starts an asyncio event loop to listen for requests on a unix domain socket. This
-    call will block until the event loop shuts down. When this function is called from
-    the main thread on macOS, the asyncio event loop uses Cocoa's CFRunLoop to process
-    event. This allows integration with Cocoa frameworks which use callbacks to process
-    use input such as clicked notifications, etc, and potentially allows showing a GUI.
+    Startup is race free: there will never be more than one daemon running with the same
+    config name. The daemon is a :class:`maestral.main.Maestral` instance which is
+    exposed as Pyro daemon object and listens for requests on a unix domain socket. This
+    call starts an asyncio event loop to process client requests and blocks until the
+    event loop shuts down. On macOS, the event loop is integrated with Cocoa's
+    CFRunLoop. This allows processing Cocoa events and callbacks, for instance for
+    desktop notifications.
 
     :param config_name: The name of the Maestral configuration to use.
-    :param log_to_stdout: If ``True``, write logs to stdout.
+    :param log_to_stderr: If ``True``, write logs to stderr.
     :param start_sync: If ``True``, start syncing once the daemon has started. If the
         ``start_sync`` call fails, an error will be logged but not raised.
     :raises RuntimeError: if a daemon for the given ``config_name`` is already running.
     """
 
     import asyncio
-    from . import notify
     from .main import Maestral
+    from .logging import scoped_logger, setup_logging
 
-    if log_to_stdout:
-        logger.setLevel(logging.DEBUG)
+    setup_logging(config_name, log_to_stderr)
+    dlogger = scoped_logger(__name__, config_name)
+
+    dlogger.info("Starting daemon")
 
     if threading.current_thread() is not threading.main_thread():
+        dlogger.error("Must run daemon in main thread")
         raise RuntimeError("Must run daemon in main thread")
 
+    dlogger.debug("Environment:\n%s", pformat(os.environ.copy()))
+
     # acquire PID lock file
     lock = maestral_lock(config_name)
 
     if lock.acquire():
-        logger.debug("Acquired daemon lock")
+        dlogger.debug("Acquired daemon lock: %s", lock.path)
     else:
-        raise RuntimeError("Maestral daemon is already running")
+        dlogger.error("Could not acquire lock, daemon is already running")
+        raise RuntimeError("Daemon is already running")
 
-    # Nice ourselves to give other processes priority. We will likely only
-    # have significant CPU usage in case of many concurrent downloads.
+    # Nice ourselves to give other processes priority.
     os.nice(10)
 
-    # integrate with CFRunLoop in macOS, only works in main thread
-    if sys.platform == "darwin":
+    # Integrate with CFRunLoop in macOS.
+    if IS_MACOS:
 
-        logger.debug("Cancelling all tasks from asyncio event loop")
+        dlogger.debug("Integrating with CFEventLoop")
 
         from rubicon.objc.eventloop import EventLoopPolicy  # type: ignore
 
-        # clean up any pending tasks before we change the event loop policy
-        # this is necessary if previous code has run an asyncio loop
-
-        loop = asyncio.get_event_loop()
-        try:
-            # Python 3.7 and higher
-            all_tasks = asyncio.all_tasks(loop)
-        except AttributeError:
-            # Python 3.6
-            all_tasks = asyncio.Task.all_tasks(loop)
-        pending_tasks = [t for t in all_tasks if not t.done()]
-
-        for task in pending_tasks:
-            task.cancel()
-
-        loop.run_until_complete(asyncio.gather(*pending_tasks, return_exceptions=True))
-        loop.close()
-
-        logger.debug("Integrating with CFEventLoop")
-
-        # set new event loop policy
         asyncio.set_event_loop_policy(EventLoopPolicy())
 
-    # get the default event loop
+    # Get the default event loop.
     loop = asyncio.get_event_loop()
 
     sd_notifier = sdnotify.SystemdNotifier()
 
-    # notify systemd that we have started
+    # Notify systemd that we have started.
     if NOTIFY_SOCKET:
-        logger.debug("Running as systemd notify service")
-        logger.debug("NOTIFY_SOCKET = %s", NOTIFY_SOCKET)
+        dlogger.debug("Running as systemd notify service")
+        dlogger.debug("NOTIFY_SOCKET = %s", NOTIFY_SOCKET)
         sd_notifier.notify("READY=1")
 
-    # notify systemd periodically if alive
+    # Notify systemd periodically if alive.
     if IS_WATCHDOG and WATCHDOG_USEC:
 
         async def periodic_watchdog() -> None:
@@ -494,25 +494,25 @@ def start_maestral_daemon(
                     sd_notifier.notify("WATCHDOG=1")
                     await asyncio.sleep(sleep / (2 * 10 ** 6))
 
-        logger.debug("Running as systemd watchdog service")
-        logger.debug("WATCHDOG_USEC = %s", WATCHDOG_USEC)
-        logger.debug("WATCHDOG_PID = %s", WATCHDOG_PID)
+        dlogger.debug("Running as systemd watchdog service")
+        dlogger.debug("WATCHDOG_USEC = %s", WATCHDOG_USEC)
+        dlogger.debug("WATCHDOG_PID = %s", WATCHDOG_PID)
         loop.create_task(periodic_watchdog())
 
-    # get socket for config name
+    # Get socket for config name.
     sockpath = sockpath_for_config(config_name)
-    logger.debug(f"Socket path: '{sockpath}'")
+    dlogger.debug(f"Socket path: '{sockpath}'")
 
-    # clean up old socket
+    # Clean up old socket.
     try:
         os.remove(sockpath)
     except FileNotFoundError:
         pass
 
-    # expose maestral as Pyro server
-    # convert management methods to one way calls so that they don't block
+    # Expose maestral as Pyro server. Convert management
+    # methods to one way calls so that they don't block.
 
-    logger.debug("Creating Pyro daemon")
+    dlogger.debug("Creating Pyro daemon")
 
     ExposedMaestral = expose(Maestral)
 
@@ -520,34 +520,28 @@ def start_maestral_daemon(
     ExposedMaestral.stop_sync = oneway(ExposedMaestral.stop_sync)
     ExposedMaestral.shutdown_daemon = oneway(ExposedMaestral.shutdown_daemon)
 
-    maestral_daemon = ExposedMaestral(config_name, log_to_stdout=log_to_stdout)
+    maestral_daemon = ExposedMaestral(config_name, log_to_stderr=log_to_stderr)
 
     if start_sync:
-
-        try:
-            maestral_daemon.start_sync()
-        except Exception as exc:
-            title = getattr(exc, "title", "Failed to start sync")
-            message = getattr(exc, "message", "Please inspect the logs")
-            logger.error(title, exc_info=True)
-            maestral_daemon.sync.notify(title, message, level=notify.ERROR)
+        dlogger.debug("Starting sync")
+        maestral_daemon.start_sync()
 
     try:
 
-        logger.debug("Starting event loop")
+        dlogger.debug("Starting event loop")
 
         with Daemon(unixsocket=sockpath) as daemon:
             daemon.register(maestral_daemon, f"maestral.{config_name}")
 
-            # reduce Pyro's housekeeping frequency from 2 sec to 20 sec
-            # this avoids constantly waking the CPU when we are idle
+            # Reduce Pyro's housekeeping frequency from 2 sec to 20 sec.
+            # This avoids constantly waking the CPU when we are idle.
             if daemon.transportServer.housekeeper:
                 daemon.transportServer.housekeeper.waittime = 20
 
             for socket in daemon.sockets:
                 loop.add_reader(socket.fileno(), daemon.events, daemon.sockets)
 
-            # handle sigterm gracefully
+            # Handle sigterm gracefully.
             signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
             for s in signals:
                 loop.add_signal_handler(s, maestral_daemon.shutdown_daemon)
@@ -557,15 +551,15 @@ def start_maestral_daemon(
             for socket in daemon.sockets:
                 loop.remove_reader(socket.fileno())
 
-            # prevent housekeeping from blocking shutdown
+            # Prevent Pyro housekeeping from blocking shutdown.
             daemon.transportServer.housekeeper = None
 
     except Exception as exc:
-        logger.error(exc.args[0], exc_info=True)
+        dlogger.error(exc.args[0], exc_info=True)
     finally:
 
         if NOTIFY_SOCKET:
-            # notify systemd that we are shutting down
+            # Notify systemd that we are shutting down.
             sd_notifier.notify("STOPPING=1")
 
 
@@ -576,9 +570,13 @@ def start_maestral_daemon_process(
 ) -> Start:
     """
     Starts the Maestral daemon in a new process by calling :func:`start_maestral_daemon`.
-    Startup is race free: there will never be two daemons running for the same config.
-    This function will use :obj:`sys.executable` as a Python executable to start the
-    daemon. Use :func:`set_executable` to use a custom executable instead.
+
+    Startup is race free: there will never be more than one daemon running for the same
+    config name. This function will use :obj:`sys.executable` as a Python executable to
+    start the daemon.
+
+    Environment variables from the current process will be preserved and updated with
+    the environment variables defined in :const:`constants.ENV`.
 
     :param config_name: The name of the Maestral configuration to use.
     :param start_sync: If ``True``, start syncing once the daemon has started.
@@ -592,7 +590,7 @@ def start_maestral_daemon_process(
     if is_running(config_name):
         return Start.AlreadyRunning
 
-    # protect against injection
+    # Protect against injection.
     cc = quote(config_name).strip("'")
     start_sync = bool(start_sync)
 
@@ -601,7 +599,10 @@ def start_maestral_daemon_process(
         f'maestral.daemon.start_maestral_daemon("{cc}", start_sync={start_sync})'
     )
 
-    cmd = [sys.executable, "-OO", "-c", script]
+    cmd = [sys.executable, "-c", script]
+
+    subprocess_env = os.environ.copy()
+    subprocess_env.update(ENV)
 
     process = subprocess.Popen(
         cmd,
@@ -609,23 +610,26 @@ def start_maestral_daemon_process(
         stdin=subprocess.DEVNULL,
         stdout=subprocess.DEVNULL,
         stderr=subprocess.DEVNULL,
+        env=subprocess_env,
     )
 
     try:
-        _wait_for_startup(config_name, timeout=timeout)
+        wait_for_startup(config_name, timeout)
     except Exception as exc:
-        logger.debug(
-            "Could not communicate with daemon",
-            exc_info=(type(exc), exc, exc.__traceback__),
-        )
+        from .logging import scoped_logger, setup_logging
 
-        # let's check what the daemon has been doing
+        setup_logging(config_name, log_to_stderr=False)
+        clogger = scoped_logger(__name__, config_name)
+
+        clogger.error("Could not communicate with daemon", exc_info=exc_info_tuple(exc))
+
+        # Let's check what the daemon has been doing.
         returncode = process.poll()
         if returncode is None:
-            logger.debug("Daemon is running but not responsive, killing now")
-            process.terminate()  # make sure we don't leave a stray process
+            clogger.error("Daemon is running but not responsive, killing now")
+            process.terminate()
         else:
-            logger.debug("Daemon stopped with return code %s", returncode)
+            clogger.error("Daemon stopped with return code %s", returncode)
         return Start.Failed
     else:
         return Start.Ok
@@ -666,7 +670,7 @@ def stop_maestral_daemon_process(
                 time.sleep(0.2)
                 timeout -= 0.2
 
-        # send SIGTERM after timeout and delete PID file
+        # Send SIGTERM after timeout and delete PID file.
         if pid:
             _send_term(pid)
 
