@@ -2,7 +2,6 @@
 """This module defines the main API which is exposed to the CLI or GUI."""
 
 # system imports
-import sys
 import os
 import os.path as osp
 import shutil
@@ -12,6 +11,9 @@ import logging.handlers
 import asyncio
 import random
 import gc
+import tempfile
+import mimetypes
+import difflib
 from concurrent.futures import ThreadPoolExecutor
 from typing import (
     Union,
@@ -24,9 +26,6 @@ from typing import (
     Optional,
     Any,
 )
-import tempfile
-import mimetypes
-import difflib
 
 # external imports
 import requests
@@ -36,15 +35,11 @@ from datetime import datetime, timezone
 from dropbox.files import FileMetadata
 from dropbox.sharing import RequestedVisibility
 
-try:
-    from systemd import journal  # type: ignore
-except ImportError:
-    journal = None
-
 # local imports
 from . import __version__
 from .client import CONNECTION_ERRORS, DropboxClient, convert_api_errors
-from .sync import SyncMonitor, SyncDirection
+from .sync import SyncDirection
+from .manager import SyncManager
 from .errors import (
     MaestralApiError,
     NotLinkedError,
@@ -55,12 +50,13 @@ from .errors import (
     UnsupportedFileTypeForDiff,
 )
 from .config import MaestralConfig, MaestralState, validate_config_name
-from .logging import CachedHandler, SdNotificationHandler, safe_journal_sender
+from .logging import CachedHandler, setup_logging, scoped_logger
 from .utils import get_newer_version
 from .utils.path import (
     is_child,
     is_equal_or_child,
-    to_existing_cased_path,
+    normalize,
+    to_existing_unnormalized_path,
     delete,
 )
 from .utils.serializer import (
@@ -70,7 +66,7 @@ from .utils.serializer import (
     StoneType,
     ErrorType,
 )
-from .utils.appdirs import get_log_path, get_cache_path
+from .utils.appdirs import get_cache_path
 from .utils.integration import get_ac_state, ACState
 from .constants import IDLE, PAUSED, CONNECTING, FileStatus, GITHUB_RELEASES_API
 
@@ -78,9 +74,6 @@ from .constants import IDLE, PAUSED, CONNECTING, FileStatus, GITHUB_RELEASES_API
 __all__ = ["Maestral"]
 
 
-logger = logging.getLogger(__name__)
-
-
 # ======================================================================================
 # Main API
 # ======================================================================================
@@ -91,8 +84,8 @@ class Maestral:
 
     All methods and properties return objects or raise exceptions which can safely be
     serialized, i.e., pure Python types. The only exception are instances of
-    :class:`errors.MaestralApiError`: they need to be registered explicitly with the
-    serpent serializer which is used for communication to frontends.
+    :class:`maestral.errors.MaestralApiError`: they need to be registered explicitly
+    with the serpent serializer which is used for communication to frontends.
 
     Sync errors and fatal errors which occur in the sync threads can be read with the
     properties :attr:`sync_errors` and :attr:`fatal_errors`, respectively.
@@ -116,16 +109,13 @@ class Maestral:
 
     :param config_name: Name of maestral configuration to run. Must not contain any
         whitespace. If the given config file does exist, it will be created.
-    :param log_to_stdout: If ``True``, Maestral will print log messages to stdout.
-        When started as a systemd services, this can result in duplicate log messages.
-        Defaults to ``False``.
+    :param log_to_stderr: If ``True``, Maestral will print log messages to stderr.
+        When started as a systemd services, this can result in duplicate log messages
+        in the systemd journal. Defaults to ``False``.
     """
 
-    log_handler_sd: Optional[SdNotificationHandler]
-    log_handler_journal: Optional["journal.JournalHandler"]
-
     def __init__(
-        self, config_name: str = "maestral", log_to_stdout: bool = False
+        self, config_name: str = "maestral", log_to_stderr: bool = False
     ) -> None:
 
         self._config_name = validate_config_name(config_name)
@@ -133,13 +123,14 @@ class Maestral:
         self._state = MaestralState(self._config_name)
 
         # set up logging
-        self._log_to_stdout = log_to_stdout
+        self._logger = scoped_logger(__name__, config_name)
+        self._log_to_stderr = log_to_stderr
         self._setup_logging()
 
         # set up sync infrastructure
         self.client = DropboxClient(config_name=self.config_name)
-        self.monitor = SyncMonitor(self.client)
-        self.sync = self.monitor.sync
+        self.manager = SyncManager(self.client)
+        self.sync = self.manager.sync
 
         self._check_and_run_post_update_scripts()
 
@@ -208,90 +199,48 @@ class Maestral:
         try:
             self.client.dbx.auth_token_revoke()
         except (ConnectionError, MaestralApiError):
-            logger.debug("Could not invalidate token with Dropbox", exc_info=True)
+            self._logger.debug("Could not invalidate token with Dropbox", exc_info=True)
 
         try:
             self.client.auth.delete_creds()
         except KeyringAccessError:
-            logger.debug("Could not remove token from keyring", exc_info=True)
+            self._logger.debug("Could not remove token from keyring", exc_info=True)
 
         # clean up config + state
         self.sync.clear_index()
         self.sync.clear_sync_history()
         self._conf.cleanup()
         self._state.cleanup()
+        self.sync._load_cached_config()  # reload cached config values
         delete(self.sync.database_path)
 
-        logger.info("Unlinked Dropbox account.")
+        self._logger.info("Unlinked Dropbox account.")
 
     def _setup_logging(self) -> None:
         """
         Sets up logging to log files, status and error properties, desktop notifications,
-        the systemd journal if available, and to stdout if requested.
+        the systemd journal if available, and to stderr if requested.
         """
+        self._root_logger = scoped_logger("maestral", self.config_name)
+        (
+            self._log_handler_file,
+            self._log_handler_stream,
+            self._log_handler_sd,
+            self._log_handler_journal,
+        ) = setup_logging(self.config_name, self._log_to_stderr)
 
-        self._logger = logging.getLogger("maestral")
-        self._logger.setLevel(logging.DEBUG)
-
-        # clean up any previous handlers
-        # TODO: use namespaced handlers for config?
-        self._logger.handlers = []
-
-        log_fmt_long = logging.Formatter(
-            fmt="%(asctime)s %(name)s %(levelname)s: %(message)s",
-            datefmt="%Y-%m-%d %H:%M:%S",
-        )
         log_fmt_short = logging.Formatter(fmt="%(message)s")
 
-        # log to file
-        log_file_path = get_log_path("maestral", f"{self._config_name }.log")
-        self.log_handler_file = logging.handlers.RotatingFileHandler(
-            log_file_path,
-            maxBytes=10 ** 7,
-            backupCount=1,
-        )
-        self.log_handler_file.setFormatter(log_fmt_long)
-        self.log_handler_file.setLevel(self.log_level)
-        self._logger.addHandler(self.log_handler_file)
-
-        # log to journal when launched from systemd
-        if journal and os.getenv("INVOCATION_ID"):
-            # noinspection PyUnresolvedReferences
-            self.log_handler_journal = journal.JournalHandler(
-                SYSLOG_IDENTIFIER="maestral", sender_function=safe_journal_sender
-            )
-            self.log_handler_journal.setFormatter(log_fmt_short)
-            self.log_handler_journal.setLevel(self.log_level)
-            self._logger.addHandler(self.log_handler_journal)
-        else:
-            self.log_handler_journal = None
-
-        # log to NOTIFY_SOCKET when launched as systemd notify service
-        if os.getenv("NOTIFY_SOCKET"):
-            self.log_handler_sd = SdNotificationHandler()
-            self.log_handler_sd.setFormatter(log_fmt_short)
-            self.log_handler_sd.setLevel(logging.INFO)
-            self._logger.addHandler(self.log_handler_sd)
-        else:
-            self.log_handler_sd = None
-
-        # log to stderr (disabled by default)
-        level = self.log_level if self._log_to_stdout else 100
-        self.log_handler_stream = logging.StreamHandler(sys.stderr)
-        self.log_handler_stream.setFormatter(log_fmt_long)
-        self.log_handler_stream.setLevel(level)
-        self._logger.addHandler(self.log_handler_stream)
-
         # log to cached handlers for status and error APIs
         self._log_handler_info_cache = CachedHandler(maxlen=1)
         self._log_handler_info_cache.setFormatter(log_fmt_short)
         self._log_handler_info_cache.setLevel(logging.INFO)
-        self._logger.addHandler(self._log_handler_info_cache)
+        self._root_logger.addHandler(self._log_handler_info_cache)
 
         self._log_handler_error_cache = CachedHandler()
         self._log_handler_error_cache.setFormatter(log_fmt_short)
         self._log_handler_error_cache.setLevel(logging.ERROR)
-        self._logger.addHandler(self._log_handler_error_cache)
+        self._root_logger.addHandler(self._log_handler_error_cache)
 
     # ==== methods to access config and saved state ====================================
 
@@ -401,15 +350,15 @@ class Maestral:
 
                     # apply changes
                     for path in added_excluded_items:
-                        logger.info("Excluded %s", path)
+                        self._logger.info("Excluded %s", path)
                         self._remove_after_excluded(path)
 
                     for path in added_included_items:
                         if not self.sync.is_excluded_by_user(path):
-                            logger.info("Included %s", path)
-                            self.monitor.added_item_queue.put(path)
+                            self._logger.info("Included %s", path)
+                            self.manager.added_item_queue.put(path)
 
-                    logger.info(IDLE)
+                    self._logger.info(IDLE)
 
                 finally:
                     self.sync.sync_lock.release()
@@ -421,52 +370,39 @@ class Maestral:
 
     @property
     def log_level(self) -> int:
-        """Log level for log files, stdout and the systemd journal."""
+        """Log level for log files, stderr and the systemd journal."""
         return self._conf.get("app", "log_level")
 
     @log_level.setter
     def log_level(self, level_num: int) -> None:
         """Setter: log_level."""
-        self.log_handler_file.setLevel(level_num)
-        if self.log_handler_journal:
-            self.log_handler_journal.setLevel(level_num)
-        if self.log_to_stdout:
-            self.log_handler_stream.setLevel(level_num)
+        self._root_logger.setLevel(min(level_num, logging.INFO))
+        self._log_handler_file.setLevel(level_num)
+        self._log_handler_stream.setLevel(level_num)
+        self._log_handler_journal.setLevel(level_num)
         self._conf.set("app", "log_level", level_num)
 
-    @property
-    def log_to_stdout(self) -> bool:
-        """Enables or disables logging to stdout."""
-        return self._log_to_stdout
-
-    @log_to_stdout.setter
-    def log_to_stdout(self, enabled: bool) -> None:
-        """Setter: log_to_stdout."""
-        self._log_to_stdout = enabled
-        level = self.log_level if enabled else 100
-        self.log_handler_stream.setLevel(level)
-
     @property
     def notification_snooze(self) -> float:
-        """Snooze time for desktop notifications in minutes. Defaults to 0.0 if
+        """Snooze time for desktop notifications in minutes. Defaults to 0 if
         notifications are not snoozed."""
-        return self.sync.notifier.snoozed
+        return self.sync.desktop_notifier.snoozed
 
     @notification_snooze.setter
     def notification_snooze(self, minutes: float) -> None:
         """Setter: notification_snooze."""
-        self.sync.notifier.snoozed = minutes
+        self.sync.desktop_notifier.snoozed = minutes
 
     @property
     def notification_level(self) -> int:
         """Level for desktop notifications. See :mod:`utils.notify` for level
         definitions."""
-        return self.sync.notifier.notify_level
+        return self.sync.desktop_notifier.notify_level
 
     @notification_level.setter
     def notification_level(self, level: int) -> None:
         """Setter: notification_level."""
-        self.sync.notifier.notify_level = level
+        self.sync.desktop_notifier.notify_level = level
 
     # ==== state information  ==========================================================
 
@@ -503,14 +439,14 @@ class Maestral:
     def paused(self) -> bool:
         """Indicates if syncing is paused by the user (read only). This is set by
         calling :meth:`pause`."""
-        return not self.monitor.autostart.is_set() and not self.sync.busy()
+        return not self.manager.autostart.is_set() and not self.sync.busy()
 
     @property
     def running(self) -> bool:
         """Indicates if sync threads are running (read only). They will be stopped
         before :meth:`start_sync` is called, when shutting down or because of an
         exception."""
-        return self.monitor.running.is_set() or self.sync.busy()
+        return self.manager.running.is_set() or self.sync.busy()
 
     @property
     def connected(self) -> bool:
@@ -519,7 +455,7 @@ class Maestral:
         if self.pending_link:
             return False
         else:
-            return self.monitor.connected
+            return self.manager.connected
 
     @property
     def status(self) -> str:
@@ -589,7 +525,13 @@ class Maestral:
 
     def get_file_status(self, local_path: str) -> str:
         """
-        Returns the sync status of an individual file.
+        Returns the sync status of a file or folder. The returned status is recursive
+        for folders, e.g., the file status will be "uploading" for a a folder if any
+        file inside that folder is being uploaded.
+
+        .. versionadded:: 1.4.4
+           Recursive behavior. Previous versions would return "up to date" even if in
+           case of syncing children.
 
         :param local_path: Path to file on the local drive. May be relative to the
             current working directory.
@@ -607,9 +549,15 @@ class Maestral:
         except ValueError:
             return FileStatus.Unwatched.value
 
-        sync_event = next(
-            iter(e for e in self.monitor.activity if e.local_path == local_path), None
-        )
+        sync_event = self.manager.activity.get(local_path)
+
+        if sync_event is None:
+            # check if there is any sync event for a child path
+            # TODO: Improve performance
+            path = next(
+                iter(p for p in self.manager.activity if p.startswith(local_path)), ""
+            )
+            sync_event = self.manager.activity.get(path)
 
         if sync_event and sync_event.direction == SyncDirection.Up:
             return FileStatus.Uploading.value
@@ -617,7 +565,7 @@ class Maestral:
             return FileStatus.Downloading.value
         elif any(dbx_path == err["dbx_path"] for err in self.sync_errors):
             return FileStatus.Error.value
-        elif self.sync.get_local_rev(dbx_path):
+        elif dbx_path == "/" or self.sync.get_local_rev(normalize(dbx_path)):
             return FileStatus.Synced.value
         else:
             return FileStatus.Unwatched.value
@@ -634,11 +582,11 @@ class Maestral:
         """
 
         self._check_linked()
-        if limit:
-            activity = [sync_event_to_dict(e) for e in self.monitor.activity[:limit]]
-        else:
-            activity = [sync_event_to_dict(e) for e in self.monitor.activity]
-        return activity
+
+        activity = list(self.manager.activity.values())[:limit]
+        serialized_activity = [sync_event_to_dict(event) for event in activity]
+
+        return serialized_activity
 
     def get_history(self, limit: Optional[int] = 100) -> List[StoneType]:
         """
@@ -655,9 +603,9 @@ class Maestral:
 
         self._check_linked()
         if limit:
-            history = [sync_event_to_dict(e) for e in self.monitor.history[-limit:]]
+            history = [sync_event_to_dict(e) for e in self.manager.history[-limit:]]
         else:
-            history = [sync_event_to_dict(e) for e in self.monitor.history]
+            history = [sync_event_to_dict(e) for e in self.manager.history]
 
         return history
 
@@ -866,13 +814,14 @@ class Maestral:
 
             return content, md
 
-        md_new = self.client.get_metadata(f"rev:{new_rev}", include_deleted=True)
+        # Get the metadata for old_rev before attempting to download. This is used
+        # to guess the file type and fail early for unsupported files.
+
         md_old = self.client.get_metadata(f"rev:{old_rev}", include_deleted=True)
 
-        if md_new is None or md_old is None:
-            missing_rev = new_rev if md_new is None else old_rev
+        if md_old is None:
             raise NotFoundError(
-                f"Could not a file with revision {missing_rev}",
+                f"Could not a file with revision {md_old}",
                 "Use 'list_revisions' to list past revisions of a file.",
             )
 
@@ -888,8 +837,11 @@ class Maestral:
                 f"Bad file type: '{mime}'", "Only files of type 'text/*' are supported."
             )
 
-        # If new_rev is None, the local file is used, even if it isn't synced
-        if new_rev is None:
+        if new_rev:
+            content_new, md_new = download_rev(new_rev)
+            date_str_new = str_from_date(md_new.client_modified)
+        else:
+            # Use the local file if new_rev is None.
             new_rev = "local version"
             try:
                 with convert_api_errors(dbx_path=dbx_path, local_path=local_path):
@@ -904,9 +856,6 @@ class Maestral:
                     "Failed to decode the file",
                     "Only UTF-8 plain text files are currently supported.",
                 )
-        else:
-            content_new, md_new = download_rev(new_rev)
-            date_str_new = str_from_date(md_new.client_modified)
 
         content_old, md_old = download_rev(old_rev)
         date_str_old = str_from_date(md_old.client_modified)
@@ -938,7 +887,7 @@ class Maestral:
 
         self._check_linked()
 
-        logger.info(f"Restoring '{dbx_path} to {rev}'")
+        self._logger.info(f"Restoring '{dbx_path} to {rev}'")
 
         res = self.client.restore(dbx_path, rev)
         return dropbox_stone_to_dict(res)
@@ -968,27 +917,20 @@ class Maestral:
         self._check_linked()
         self._check_dropbox_dir()
 
-        self.monitor.rebuild_index()
+        self.manager.rebuild_index()
 
     def start_sync(self) -> None:
         """
         Creates syncing threads and starts syncing.
-
-        :raises NotLinkedError: if no Dropbox account is linked.
-        :raises NoDropboxDirError: if local Dropbox folder is not set up.
         """
-
-        self._check_linked()
-        self._check_dropbox_dir()
-
-        self.monitor.start()
+        self.manager.start()
 
     def stop_sync(self) -> None:
         """
         Stops all syncing threads if running. Call :meth:`start_sync` to restart
         syncing.
         """
-        self.monitor.stop()
+        self.manager.stop()
 
     def reset_sync_state(self) -> None:
         """
@@ -1000,7 +942,7 @@ class Maestral:
         """
 
         self._check_linked()
-        self.monitor.reset_sync_state()
+        self.manager.reset_sync_state()
 
     def set_excluded_items(self, items: List[str]) -> None:
         warnings.warn(
@@ -1039,8 +981,8 @@ class Maestral:
             )
 
         if self.sync.is_excluded_by_user(dbx_path):
-            logger.info("%s was already excluded", dbx_path)
-            logger.info(IDLE)
+            self._logger.info("%s was already excluded", dbx_path)
+            self._logger.info(IDLE)
             return
 
         if self.sync.sync_lock.acquire(blocking=False):
@@ -1058,8 +1000,8 @@ class Maestral:
 
                 self._remove_after_excluded(dbx_path)
 
-                logger.info("Excluded %s", dbx_path)
-                logger.info(IDLE)
+                self._logger.info("Excluded %s", dbx_path)
+                self._logger.info(IDLE)
             finally:
                 self.sync.sync_lock.release()
 
@@ -1076,12 +1018,12 @@ class Maestral:
         local_path = self.sync.to_local_path_from_cased(dbx_path)
         # dbx_path will be lower-case, we there explicitly run `to_existing_cased_path`
         try:
-            local_path = to_existing_cased_path(local_path)
+            local_path = to_existing_unnormalized_path(local_path)
         except FileNotFoundError:
             pass
         else:
             event_cls = DirDeletedEvent if osp.isdir(local_path) else FileDeletedEvent
-            with self.monitor.sync.fs_events.ignore(event_cls(local_path)):
+            with self.manager.sync.fs_events.ignore(event_cls(local_path)):
                 delete(local_path)
 
     def include_item(self, dbx_path: str) -> None:
@@ -1123,8 +1065,8 @@ class Maestral:
             )
 
         if not self.sync.is_excluded_by_user(dbx_path):
-            logger.info("'%s' is already included, nothing to do", dbx_path)
-            logger.info(IDLE)
+            self._logger.info("'%s' is already included, nothing to do", dbx_path)
+            self._logger.info(IDLE)
             return
 
         # ---- update excluded items list ----------------------------------------------
@@ -1166,11 +1108,11 @@ class Maestral:
                 # ---- download item from Dropbox --------------------------------------
 
                 if excluded_parent:
-                    logger.info("Included '%s' and parent directories", dbx_path)
-                    self.monitor.added_item_queue.put(excluded_parent)
+                    self._logger.info("Included '%s' and parent directories", dbx_path)
+                    self.manager.added_item_queue.put(excluded_parent)
                 else:
-                    logger.info("Included '%s'", dbx_path)
-                    self.monitor.added_item_queue.put(dbx_path)
+                    self._logger.info("Included '%s'", dbx_path)
+                    self.manager.added_item_queue.put(dbx_path)
             finally:
                 self.sync.sync_lock.release()
 
@@ -1213,7 +1155,7 @@ class Maestral:
         self._check_linked()
         self._check_dropbox_dir()
 
-        logger.info("Moving Dropbox folder...")
+        self._logger.info("Moving Dropbox folder...")
 
         # input checks
         old_path = self.sync.dropbox_path
@@ -1221,7 +1163,7 @@ class Maestral:
 
         try:
             if osp.samefile(old_path, new_path):
-                logger.info(f'Dropbox folder moved to "{new_path}"')
+                self._logger.info(f'Dropbox folder moved to "{new_path}"')
                 return
         except FileNotFoundError:
             pass
@@ -1242,7 +1184,7 @@ class Maestral:
         # update config file and client
         self.sync.dropbox_path = new_path
 
-        logger.info(f'Dropbox folder moved to "{new_path}"')
+        self._logger.info(f'Dropbox folder moved to "{new_path}"')
 
         # resume syncing
         if was_syncing:
@@ -1268,7 +1210,7 @@ class Maestral:
 
         # housekeeping
         path = osp.realpath(osp.expanduser(path))
-        self.monitor.reset_sync_state()
+        self.manager.reset_sync_state()
 
         # create new folder
         os.makedirs(path, exist_ok=True)
@@ -1377,7 +1319,7 @@ class Maestral:
         self._check_linked()
         self._check_dropbox_dir()
 
-        return self.sync.to_local_path_from_cased(dbx_path)
+        return self.sync.to_local_path(dbx_path)
 
     def check_for_updates(self) -> Dict[str, Union[str, bool, None]]:
         """
@@ -1393,8 +1335,10 @@ class Maestral:
         error_msg = None
 
         try:
-            r = requests.get(GITHUB_RELEASES_API)
-            data = r.json()
+            resp = requests.get(GITHUB_RELEASES_API)
+            resp.raise_for_status()
+
+            data = resp.json()
 
             releases = []
             release_notes = []
@@ -1456,7 +1400,8 @@ class Maestral:
 
         if not self.client.linked:
             raise NotLinkedError(
-                "No Dropbox account linked", 'Please call "link" to link an account.'
+                "No Dropbox account linked",
+                "Please link an account using the GUI or CLI.",
             )
 
     def _check_dropbox_dir(self) -> None:
@@ -1464,7 +1409,7 @@ class Maestral:
         if self.pending_dropbox_folder:
             raise NoDropboxDirError(
                 "No local Dropbox directory",
-                'Call "create_dropbox_directory" to set up.',
+                "Please set up a local Dropbox directory using the GUI or CLI.",
             )
 
     # ==== housekeeping on update  =====================================================
@@ -1478,8 +1423,10 @@ class Maestral:
 
         if Version(updated_from) < Version("1.2.1"):
             self._update_from_pre_v1_2_1()
-        elif Version(updated_from) < Version("1.3.2"):
+        if Version(updated_from) < Version("1.3.2"):
             self._update_from_pre_v1_3_2()
+        if Version(updated_from) < Version("1.4.5"):
+            self._update_from_pre_v1_4_5()
 
         self.set_state("app", "updated_scripts_completed", __version__)
 
@@ -1489,9 +1436,13 @@ class Maestral:
     def _update_from_pre_v1_3_2(self) -> None:
 
         if self._conf.get("app", "keyring") == "keyring.backends.OS_X.Keyring":
-            logger.info("Migrating keyring after update from pre v1.3.2")
+            self._logger.info("Migrating keyring after update from pre v1.3.2")
             self._conf.set("app", "keyring", "keyring.backends.macOS.Keyring")
 
+    def _update_from_pre_v1_4_5(self) -> None:
+        # clear sync history table because we have added new columns
+        self.sync.clear_sync_history()
+
     # ==== period async jobs ===========================================================
 
     def _schedule_task(self, coro: Awaitable) -> None:
@@ -1541,16 +1492,16 @@ class Maestral:
 
         while True:
 
-            if self.monitor.running.is_set():
+            if self.manager.running.is_set():
                 elapsed = time.time() - self.sync.last_reindex
                 ac_state = get_ac_state()
 
-                reindexing_due = elapsed > self.monitor.reindex_interval
-                is_idle = self.monitor.idle_time > 20 * 60
+                reindexing_due = elapsed > self.manager.reindex_interval
+                is_idle = self.manager.idle_time > 20 * 60
                 has_ac_power = ac_state in (ACState.Connected, ACState.Undetermined)
 
                 if reindexing_due and is_idle and has_ac_power:
-                    self.monitor.rebuild_index()
+                    self.manager.rebuild_index()
 
             await sleep_rand(60 * 5)
 
