@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 """
-This module handles desktop notifications and supports multiple backends, depending on
-the platform.
+This module handles desktop notifications It uses the
+`desktop-notifier <https://desktop-notifier.readthedocs.io/en/latest/>`_ package as a
+backend for cross-platform desktop notifications.
 """
 
 # system imports
@@ -34,10 +35,13 @@ _desktop_notifier = DesktopNotifier(
 
 
 NONE = 100
+"""No desktop notifications"""
 ERROR = 40
+"""Notify only on fatal errors"""
 SYNCISSUE = 30
+"""Notify for sync issues and higher"""
 FILECHANGE = 15
-
+"""Notify for all remote file changes"""
 
 _level_to_name = {
     NONE: "NONE",
@@ -55,7 +59,12 @@ _name_to_level = {
 
 
 def level_number_to_name(number: int) -> str:
-    """Converts a Maestral notification level number to name."""
+    """
+    Converts a Maestral notification level number to name.
+
+    :param number: Level number.
+    :returns: Level name.
+    """
 
     try:
         return _level_to_name[number]
@@ -64,7 +73,12 @@ def level_number_to_name(number: int) -> str:
 
 
 def level_name_to_number(name: str) -> int:
-    """Converts a Maestral notification level name to number."""
+    """
+    Converts a Maestral notification level name to number.
+
+    :param name: Level name.
+    :returns: Level number.
+    """
 
     try:
         return _name_to_level[name]
@@ -76,6 +90,9 @@ class MaestralDesktopNotifier:
     """Desktop notification emitter for Maestral
 
     Desktop notifier with snooze functionality and variable notification levels.
+
+    :param config_name: Config name. This is used to access notification settings for
+        the daemon.
     """
 
     def __init__(self, config_name: str) -> None:
